@@ -1,10 +1,11 @@
+from collections.abc import Mapping
 import os, io, sys, math, json, base64, unittest
 from random import Random
 from typing import Any, Literal, cast
 from functools import partial
 
 from .typing import (
-    MISSING, PRESENT, FridBeing, FridMixin, FridValue, FridNameArgs, StrKeyMap, ValueArgs,
+    MISSING, PRESENT, FridBeing, FridError, FridMixin, FridValue, FridNameArgs, StrKeyMap, ValueArgs,
     get_func_name, get_type_name, get_qual_name
 )
 from .chrono import DateTimeDiff, DateTimeSpec, parse_datetime, parse_timeonly, strfr_datetime
@@ -426,7 +427,7 @@ class TestLoadsAndDumps(unittest.TestCase):
                 self.assertEqual(data, load_frid_str(s, json_level=json_level, **load_args),
                                  msg=f"{json_level=} {len(s)=}")
             # Test with only all possible frid values
-            json_level = rng.choice([0, 1, 5])
+            json_level: Literal[0,1,5] = rng.choice([0, 1, 5])
             for escape_seq in ('~', "#!"):
                 data = frid_random(rng, tree, for_json=0)
                 s = dump_frid_str(data, json_level=json_level,
@@ -439,22 +440,22 @@ class TestLoadsAndDumps(unittest.TestCase):
                               escape_seq=escape_seq, **dump_args)
                 self.assertEqual(s, t.getvalue())
                 self.assertEqual(data, load_frid_tio(
-                    io.StringIO(s), page=rng.randint(1, 5), json_level=1, escape_seq=escape_seq,
+                    io.StringIO(s), rng.randint(1, 5), json_level=1, escape_seq=escape_seq,
                     **load_args
                 ), msg=f"{len(s)=}")
                 # With loose mode
                 self.assertEqual(data, load_frid_tio(
-                    io.StringIO(s), page=rng.randint(1, 5), json_level=1, escape_seq=escape_seq,
+                    io.StringIO(s), rng.randint(1, 5), json_level=1, escape_seq=escape_seq,
                     loose_mode=True, **load_args
                 ), msg=f"{len(s)=}")
-                # With indentitation and newlines
+                # With indentitation and lineends
                 s = dump_frid_str(data, json_level=json_level, indent=4,
                                   escape_seq=escape_seq, **dump_args)
                 self.assertEqual(data, load_frid_str(
                     s, json_level=json_level, escape_seq=escape_seq, **load_args
                 ), msg=f"{len(s)=}")
                 self.assertEqual(data, load_frid_tio(
-                    io.StringIO(s), page=64, json_level=json_level, escape_seq=escape_seq,
+                    io.StringIO(s), 64, json_level=json_level, escape_seq=escape_seq,
                     **load_args
                 ), msg=f"{len(s)=}")
 
@@ -488,7 +489,7 @@ class TestLoadsAndDumps(unittest.TestCase):
     def test_negative_json_dump(self):
         for i, k in enumerate(self.negative_json_dump_cases):
             with self.assertRaises(ValueError, msg=f"[{i}]: {k}"):
-                dump_frid_str(k, json_level=True)
+                dump_frid_str(k, json_level=1)
 
     expression_cases = {
         "()": "", "(())": "()", "(')')": "')'",
@@ -496,7 +497,7 @@ class TestLoadsAndDumps(unittest.TestCase):
     }
     def test_expression(self):
         for i, (k, v) in enumerate(self.expression_cases.items()):
-            w = load_frid_str(k, parse_expr=lambda x, p: x)
+            w = load_frid_str(k, parse_expr=(lambda x, p: x))
             self.assertEqual(v, w, msg=f"[{i}]: {k}")
 
     comment_cases = {
@@ -539,7 +540,7 @@ class TestLoadsAndDumps(unittest.TestCase):
             super().__init__()
             self._args = args
             self._kwds = kwds
-        def frid_repr(self) -> FridNameArgs:
+        def frid_repr(self, **kwargs) -> FridNameArgs:
             return FridNameArgs(get_type_name(self), self._args, self._kwds)
         def __repr__(self):
             return dump_args_str(self.frid_repr())
@@ -550,20 +551,19 @@ class TestLoadsAndDumps(unittest.TestCase):
                 return False
             return self._args == other._args and self._kwds == other._kwds
     class TestMixinClass2(TestMixinClass):
-        def __init__(self, opt1, *args, opt2, **kwds):
+        def __init__(self, *args, opt, **kwds):
             super().__init__(*args, **kwds)
-            self._opt1 = opt1
-            self._opt2 = opt2
+            self._opt = opt
         @classmethod
-        def frid_from(cls, data: FridNameArgs, opt1, /, *, opt2):
-            return cls(opt1, *data.args, opt2=opt2, **data.kwds)
-        def frid_repr(self, opt1, /, *, opt2) -> FridNameArgs:
-            assert opt1 == self._opt1 and opt2 == self._opt2
+        def frid_from(cls, data: FridNameArgs, /, *, opt, **kwargs):
+            return cls(*data.args, opt=opt, **data.kwds)
+        def frid_repr(self, *, opt, **kwargs) -> FridNameArgs:
+            assert opt == self._opt
             return super().frid_repr()
         def __eq__(self, other):
             if not super().__eq__(other):
                 return False
-            return self._opt1 == other._opt1 and self._opt2 == other._opt2
+            return self._opt == other._opt
 
     def test_mixins(self):
         test = self.TestMixinClass()
@@ -586,18 +586,25 @@ class TestLoadsAndDumps(unittest.TestCase):
             json, frid_mixin=[self.TestMixinClass], json_level=1, escape_seq="#!"
         ), test)
 
-        test = self.TestMixinClass2("1", "Test", opt2=2, a=3)
+        test = self.TestMixinClass2("Test", opt=2, a=3)
         frid = "TestMixinClass2(Test, a=3)"
-        mixin_list = [ValueArgs(self.TestMixinClass2, "1", opt2=2)]
-        self.assertEqual(dump_frid_str(test, mixin_args=mixin_list), frid)
-        self.assertEqual(load_frid_str(frid, frid_mixin=mixin_list), test)
+        value_args: ValueArgs[type[FridMixin]] = ValueArgs(self.TestMixinClass2, opt=2)
+        self.assertEqual(dump_frid_str(test, mixin_args=[value_args]), frid)
+        self.assertEqual(load_frid_str(frid, frid_mixin=[value_args]), test)
         json = """{"": ["#!TestMixinClass2", "Test"], "a": 3}"""
-        self.assertEqual(dump_frid_str(test, mixin_args=mixin_list,
+        self.assertEqual(dump_frid_str(test, mixin_args=[value_args],
                                        json_level=1, escape_seq="#!"), json)
         self.assertEqual(load_frid_str(
-            json, frid_mixin=mixin_list, json_level=1, escape_seq="#!"
+            json, frid_mixin=[value_args], json_level=1, escape_seq="#!"
         ), test)
 
+    def test_frid_error(self):
+        data = {'error': FridError("Test")}
+        s = dump_frid_str(data)
+        self.assertEqual(s, "{error: FridError(error=Test)}")
+        t = load_frid_str(s, frid_mixin=[FridError])
+        self.assertTrue(isinstance(t, Mapping) and isinstance(t.get('error'), FridError))
+
     def test_identation(self):
         data = {'x': ["a", "b", []], 'y': 2, 'z': self.TestMixinClass(3, 4, b=5)}
         s = ("{\n\tx: [\n\t\ta,\n\t\tb,\n\t\t[\n\t\t]?\n\t],\n"
