@@ -1,9 +1,10 @@
 import math
 from abc import ABC, abstractmethod
 from collections.abc import Callable, Iterable, Mapping, Sequence, Set
-from typing import Any, Literal, TextIO
+from typing import Any, Literal, TextIO, TypedDict
 from enum import Enum, auto
 
+from .typing import Unpack
 from .typing import MISSING, PRESENT, FridBasic, FridBeing, BlobTypes, FridNameArgs, ValueArgs
 from .typing import FridMixin, FridValue, StrKeyMap
 from .chrono import DateTypes, strfr_datetime
@@ -70,15 +71,21 @@ class PPToTextIOMixin(PrettyPrint):
         self.stream.write(token)
 
 class MultilineFormatMixin(PrettyPrint):
-    def __init__(self, *args, indent: int|str|None=None, extra_comma=False,
-                 break_quoted: bool=False,
-                 newline: str='\n', newline_after: str="{[", **kwargs):
-        super().__init__(*args, **kwargs)
-        self.indent = ' ' * indent if isinstance(indent, int) else indent
-        self.newline = newline
-        self.extra_comma = extra_comma
-        self.break_quoted = break_quoted
-        self.newline_after = newline_after
+    class PPParams(TypedDict, total=False):
+        indent: int|str
+        extra_comma: bool
+        break_quoted: bool
+        newline: str
+        newline_after: str
+    def __init__(self, *args, **kwargs: Unpack[PPParams]):
+        self.indent = ' ' * indent if isinstance((
+            indent := kwargs.pop('indent', None)
+        ), int) else indent
+        self.newline = kwargs.pop('newline', '\n')
+        self.extra_comma = kwargs.pop('extra_comma', False)
+        self.break_quoted = kwargs.pop('break_quoted', False)
+        self.newline_after = kwargs.pop('newline_after', "{[")
+        super().__init__(*args, **kwargs)  # type: ignore  -- No deconstructor/unpacker for map
         self._level = 0
         self._delta: list[bool] = []
         self._indented_back = False
@@ -175,36 +182,38 @@ class FridDumper(PrettyPrint):
     - `print_user`: a user callback to convert any unrecognized data types to string.
     - Other constructor parameter as supported by `PrettyPrint` class
     """
-    def __init__(self, *args, json_level: Literal[0,1,5]=0, escape_seq: str|None=None,
-                 ascii_only: bool=False,
-                 mixin_args: Iterable[ValueArgs[type[FridMixin]]]|None=None,
-                 print_real: Callable[[int|float],str|None]|None=None,
-                 print_date: Callable[[DateTypes],str|None]|None=None,
-                 print_blob: Callable[[BlobTypes],str|None]|None=None,
-                 print_user: Callable[[Any,str],str|None]|None=None,
-                 **kwargs):
-        super().__init__(*args, **kwargs)
-        self.json_level = json_level
-        self.escape_seq = escape_seq
-        self.ascii_only = ascii_only
-        self.print_real = print_real
-        self.print_date = print_date
-        self.print_blob = print_blob
-        self.print_user = print_user
+    class Params(TypedDict, total=False):
+        json_level: Literal[0,1,5]
+        escape_seq: str
+        ascii_only: bool
+        mixin_args: Iterable[ValueArgs[type[FridMixin]]]
+        print_real: Callable[[int|float],str|None]
+        print_date: Callable[[DateTypes],str|None]
+        print_blob: Callable[[BlobTypes],str|None]
+        print_user: Callable[[Any,str],str|None]
+    def __init__(self, *args, **kwargs: Unpack[Params]):
+        self.json_level = kwargs.pop('json_level', 0)
+        self.escape_seq = kwargs.pop('escape_seq', None)
+        self.ascii_only = kwargs.pop('ascii_only', False)
+        self.print_real = kwargs.pop('print_real', None)
+        self.print_date = kwargs.pop('print_date', None)
+        self.print_blob = kwargs.pop('print_blob', None)
+        self.print_user = kwargs.pop('print_user', None)
         self.mixin_args: dict[type[FridMixin],ValueArgs[type[FridMixin]]] = {}
-        if mixin_args:
+        if (mixin_args := kwargs.pop('mixin_args', None)):
             for item in mixin_args:
                 self.mixin_args[item.data] = item
+        super().__init__(*args, **kwargs)  # type: ignore -- wait until dict unpack in Python
         if not self.json_level:
             pairs = EXTRA_ESCAPE_PAIRS
             hex_prefix = ('x', 'u', 'U')
-        elif json_level == 5:
+        elif self.json_level == 5:
             pairs = JSON5_ESCAPE_PAIRS
             hex_prefix = ('x', 'u', None)
         else:
             pairs = JSON1_ESCAPE_PAIRS
             hex_prefix = (None, 'u', None)
-        if ascii_only:
+        if self.ascii_only:
             self.se_encoder = StringEscapeEncode(pairs, '\\')
         else:
             self.se_encoder = StringEscapeEncode(pairs, '\\', hex_prefix)
@@ -459,6 +468,9 @@ class FridDumper(PrettyPrint):
         else:
             raise ValueError(f"Invalid type {type(data)} for json={self.json_level} at {path=}")
 
+class FridMultilineDumperParams(FridDumper.Params, MultilineFormatMixin.PPParams, total=False):
+    pass
+
 class FridStringDumper(PPToStringMixin, MultilineFormatMixin, FridDumper):
     pass
 
@@ -466,24 +478,25 @@ class FridTextIODumper(PPToTextIOMixin, MultilineFormatMixin, FridDumper):
     pass
 
 def dump_frid_str(data: FridValue, /, *args, init_path: str='',
-                  top_delim: bool=True, **kwargs) -> str:
+                  top_delim: bool=True, **kwargs: Unpack[FridMultilineDumperParams]) -> str:
     dumper = FridStringDumper(*args, **kwargs)
     dumper.print_frid_value(data, init_path, top_delim=top_delim)
     return str(dumper)
 
 def dump_frid_tio(data: FridValue, /, file: TextIO, *args, init_path: str='',
-                  top_delim: bool=True, **kwargs) -> TextIO:
+                  top_delim: bool=True, **kwargs: Unpack[FridMultilineDumperParams]) -> TextIO:
     dumper = FridTextIODumper(file, *args, **kwargs)
     dumper.print_frid_value(data, init_path, top_delim=top_delim)
     return file
 
-def dump_args_str(named_args: FridNameArgs, *args, **kwargs) -> str:
+def dump_args_str(named_args: FridNameArgs, *args,
+                  **kwargs: Unpack[FridMultilineDumperParams]) -> str:
     dumper = FridStringDumper(*args, **kwargs)
     dumper.print_named_args(named_args, '')
     return str(dumper)
 
-def dump_args_tio(named_args: FridNameArgs, /, file: TextIO, *args, **kwargs) -> TextIO:
+def dump_args_tio(named_args: FridNameArgs, /, file: TextIO, *args,
+                  **kwargs: Unpack[FridMultilineDumperParams]) -> TextIO:
     dumper = FridTextIODumper(file, *args, **kwargs)
     dumper.print_named_args(named_args, '')
     return file
-
