@@ -5,9 +5,10 @@ import asyncio, threading
 from dataclasses import dataclass, field
 from abc import abstractmethod
 from collections.abc import Callable, Iterable, Mapping
-from typing import Any, Concatenate, Generic, ParamSpec, TypeVar
+from typing import Any, Concatenate, Generic, ParamSpec, TypeVar, TypedDict
 from urllib.parse import urlparse
 
+from ..typing import Unpack
 from ..typing import MISSING, PRESENT, BlobTypes, MissingType, frid_type_size
 from ..typing import FridTypeName, FridTypeSize, FridValue, FridArray, FridBeing, StrKeyMap
 from ..aio import CountedAsyncLock
@@ -15,8 +16,8 @@ from ..chrono import parse_datetime, strfr_datetime, datetime, timezone
 from ..guards import is_frid_array, is_frid_skmap
 from .._basic import frid_mingle
 from ..lib.texts import str_encode_nonprints, str_decode_nonprints
-from .._dumps import dump_frid_str
-from .._loads import load_frid_str
+from .._dumps import dump_frid_str, FridDumper
+from .._loads import load_frid_str, FridLoader
 from .store import AsyncStore, ValueStore
 from .utils import KeySearch, VSPutFlag, VStoreKey, VStoreSel, frid_delete, frid_select, list_concat, match_key
 
@@ -36,14 +37,14 @@ class _SimpleBaseStore(Generic[_E]):
         """Decodes the data from a generic encoded `val` (bytes, string, etc)."""
         raise NotImplementedError
 
-    def _get(self, key: str, /) -> _E|MissingType:
+    def _get(self, key: str, /):
         """Get the whole data from the store associated to the given `key`."""
         raise NotImplementedError  # pragma: no cover
-    def _put(self, key: str, val: _E, /) -> bool:
+    def _put(self, key: str, val: _E, /):
         """Write the whole data into the store associated to the given `key`."""
         raise NotImplementedError  # pragma: no cover
     def _rmw(self, key: str, mod: ModFunc[_E,_P],
-             /, flags: VSPutFlag, *args: _P.args, **kwargs: _P.kwargs) -> bool:
+             /, flags: VSPutFlag, *args: _P.args, **kwargs: _P.kwargs):
         """The read-modify-write process for the value of the `key` in the store.
         - `mod`: the callback function to be called with:
             + The current value as the first argument (or MISSING);
@@ -56,7 +57,7 @@ class _SimpleBaseStore(Generic[_E]):
         - This method returns True iff the storage is changed.
         """
         raise NotImplementedError  # pragma: no cover
-    def _del(self, key: str, /) -> bool:
+    def _del(self, key: str, /):
         """Delete the data in the store associated to the given `key`.
         - Returns boolean to indicate if the key is deleted (or if the store is changed).
         """
@@ -87,7 +88,7 @@ class _SimpleBaseStore(Generic[_E]):
         if new_data is old_data:
             return PRESENT
         return self._encode(new_data)
-    def _del_sel(self, val: _E|MissingType, sel: VStoreSel) -> _E|FridBeing:
+    def _del_sel(self, val: _E|MissingType, sel: VStoreSel, /) -> _E|FridBeing:
         """Deletes the selected items in general.
         - Return he updated value (with PRESENT for no change and MISSING to delete entry).
         """
@@ -104,17 +105,17 @@ class _SimpleBaseStore(Generic[_E]):
 class SimpleValueStore(_SimpleBaseStore[_E], ValueStore):
     """This is the base class of simple value store that loads and saves records in full."""
     @abstractmethod
-    def _get(self, key: str) -> _E|MissingType:
+    def _get(self, key: str, /) -> _E|MissingType:
         raise NotImplementedError  # pragma: no cover
     @abstractmethod
-    def _put(self, key: str, val: _E) -> bool:
+    def _put(self, key: str, val: _E, /) -> bool:
         raise NotImplementedError  # pragma: no cover
     @abstractmethod
     def _rmw(self, key: str, mod: ModFunc[_E,_P],
              /, flags: VSPutFlag, *args: _P.args, **kwargs: _P.kwargs) -> bool:
         raise NotImplementedError  # pragma: no cover
     @abstractmethod
-    def _del(self, key: str) -> bool:
+    def _del(self, key: str, /) -> bool:
         raise NotImplementedError  # pragma: no cover
 
     def get_frid(self, key: VStoreKey, sel: VStoreSel=None,
@@ -144,17 +145,17 @@ class SimpleValueStore(_SimpleBaseStore[_E], ValueStore):
 class SimpleAsyncStore(_SimpleBaseStore[_E], AsyncStore):
     """This is the base class of simple async store that loads and saves records in full."""
     @abstractmethod
-    async def _get(self, key: str) -> _E|MissingType:
+    async def _get(self, key: str, /) -> _E|MissingType:
         raise NotImplementedError  # pragma: no cover
     @abstractmethod
-    async def _put(self, key: str, val: _E) -> bool:
+    async def _put(self, key: str, val: _E, /) -> bool:
         raise NotImplementedError  # pragma: no cover
     @abstractmethod
     async def _rmw(self, key: str, mod: ModFunc[_E,_P],
                    /, flags: VSPutFlag, *args: _P.args, **kwargs: _P.kwargs) -> bool:
         raise NotImplementedError  # pragma: no cover
     @abstractmethod
-    async def _del(self, key: str) -> bool:
+    async def _del(self, key: str, /) -> bool:
         raise NotImplementedError  # pragma: no cover
 
     async def get_frid(self, key: VStoreKey, sel: VStoreSel=None,
@@ -192,7 +193,7 @@ class MemoryValueStore(SimpleValueStore[FridValue]):
         alock: asyncio.Lock = field(default_factory=CountedAsyncLock)
 
     DataSpaceType = dict[tuple[str,...],StoreMeta]
-    def __init__(self, dataspace: DataSpaceType|None=None, namespace: tuple[str,...]=(), /):
+    def __init__(self, dataspace: DataSpaceType|None=None, /, namespace: tuple[str,...]=()):
         super().__init__()
         self._storage = dataspace if dataspace is not None else self.DATA_SPACE
         self._meta = self._storage.setdefault(namespace, self.StoreMeta())
@@ -201,20 +202,20 @@ class MemoryValueStore(SimpleValueStore[FridValue]):
         return self._data
 
     @classmethod
-    def from_url(cls, url: str, **kwargs) -> 'MemoryValueStore':
+    def from_url(cls, url: str, *, namespace: tuple[str,...]=()) -> 'MemoryValueStore':
         # Allow passing an URL through but the content is not checked
         # This function allows a path; for example memory:///abc/def means namespace=(abc, def)
         result = urlparse(url)
         if result.scheme != cls.URL_SCHEME:
             raise ValueError("Unsupported URL scheme for memory value store: {result.scheme}")
         if (path := result.path.strip('/')):
-            namespace = path.split('/')
-            if (value := kwargs.get('namespace')):
-                namespace.extend(value)
-            kwargs['namespace'] = tuple(namespace)
+            ns = path.split('/')
+            if namespace:
+                ns.extend(namespace)
+            namespace = tuple(ns)
         if result.netloc or result.query or result.fragment:
             raise ValueError("The URL should just be {cls.URL_SCHEME}://[/name/space]")
-        return cls(**kwargs)
+        return cls(namespace=namespace)
     def substore(self, name: str, *args: str) -> 'MemoryValueStore':
         return __class__(self._storage, (name, *args))
 
@@ -287,26 +288,34 @@ class BinaryStoreMixin:
     Also _insert_prefix() and _remove_prefix() can be overridden to handle
     prefix matching add/or to extra information after the prefix.
     """
-    def __init__(self, *, frid_prefix: bytes=b'',
-                 text_prefix: bytes|None=None, blob_prefix: bytes|None=None,
-                 list_prefix: bytes|None=None, dict_prefix: bytes|None=None, **kwargs):
-        super().__init__(**kwargs)
-        self._frid_prefix = frid_prefix
-        self._text_prefix = text_prefix
-        self._blob_prefix = blob_prefix
-        self._list_prefix = list_prefix
-        self._dict_prefix = dict_prefix
+    class Params(TypedDict, total=False):
+        frid_prefix: bytes
+        text_prefix: bytes
+        blob_prefix: bytes
+        list_prefix: bytes
+        dict_prefix: bytes
+        frid_loader_params: FridLoader.Params
+        frid_dumper_params: FridDumper.Params
+    def __init__(self, **kwargs: Unpack[Params]):
+        self._frid_prefix = kwargs.pop('frid_prefix', b'')
+        self._text_prefix = kwargs.pop('text_prefix', None)
+        self._blob_prefix = kwargs.pop('blob_prefix', None)
+        self._list_prefix = kwargs.pop('list_prefix', None)
+        self._dict_prefix = kwargs.pop('dict_prefix', None)
+        self._frid_loader_params: FridLoader.Params = kwargs.pop('frid_loader_params', {})
+        self._frid_dumper_params: FridDumper.Params = kwargs.pop('frid_dumper_params', {})
+        super().__init__(**kwargs)  # type: ignore --- need dict Unpacker
         decoders: list[tuple[bytes,Callable[[bytes],FridValue]]] = []
-        if frid_prefix is not None:
-            decoders.append((frid_prefix, self._decode_frid))
-        if text_prefix is not None:
-            decoders.append((text_prefix, self._decode_text))
-        if blob_prefix is not None:
-            decoders.append((blob_prefix, self._decode_blob))
-        if list_prefix is not None:
-            decoders.append((list_prefix, self._decode_list))
-        if dict_prefix is not None:
-            decoders.append((dict_prefix, self._decode_dict))
+        if self._frid_prefix is not None:
+            decoders.append((self._frid_prefix, self._decode_frid))
+        if self._text_prefix is not None:
+            decoders.append((self._text_prefix, self._decode_text))
+        if self._blob_prefix is not None:
+            decoders.append((self._blob_prefix, self._decode_blob))
+        if self._list_prefix is not None:
+            decoders.append((self._list_prefix, self._decode_list))
+        if self._dict_prefix is not None:
+            decoders.append((self._dict_prefix, self._decode_dict))
         decoders.sort(reverse=True, key=lambda x: len(x[0]))
         self._decoders = dict(decoders)
         if len(self._decoders) < len(decoders):
@@ -385,7 +394,7 @@ class BinaryStoreMixin:
         """Encodes general frid-supported data.
         - This method is used no specific encoding is specified.
         """
-        return dump_frid_str(data).encode()
+        return dump_frid_str(data, **self._frid_dumper_params).encode()
     def _encode_blob(self, data: BlobTypes, /) -> bytes:
         """Encodes blob (binary data)."""
         return bytes(data)
@@ -394,7 +403,8 @@ class BinaryStoreMixin:
         return data.encode()
     def _encode_list(self, data: FridArray, /) -> bytes:
         """Encodes a list as lines."""
-        out: list[bytes] = [dump_frid_str(item).encode() for item in data]
+        out: list[bytes] = [dump_frid_str(item, **self._frid_dumper_params).encode()
+                            for item in data]
         out.append(b'')
         return b'\n'.join(out)
     def _encode_dict(self, data: StrKeyMap, /) -> bytes:
@@ -406,7 +416,8 @@ class BinaryStoreMixin:
         out: list[bytes] = []
         for k, v in data.items():
             line = str_encode_nonprints(k, '\x7f')
-            line += "\t" + (v.strfr() if isinstance(v, FridBeing) else dump_frid_str(v))
+            line += "\t" + (v.strfr() if isinstance(v, FridBeing)
+                            else dump_frid_str(v, **self._frid_dumper_params))
             out.append(line.encode())
         out.append(b'')
         return b'\n'.join(out)
@@ -435,7 +446,7 @@ class BinaryStoreMixin:
         raise ValueError(f"Invalid byte encoding of {len(val)} bytes")
     def _decode_frid(self, val: bytes, /) -> FridValue:
         """Decode the value as the generic frid representation."""
-        return load_frid_str(val.decode())
+        return load_frid_str(val.decode(), **self._frid_dumper_params)
     def _decode_text(self, val: bytes, /) -> str:
         """Decode the value as the string representation."""
         return val.decode()
@@ -444,7 +455,8 @@ class BinaryStoreMixin:
         return val
     def _decode_list(self, val: bytes, /) -> FridArray:
         """Decode the value as the representation for a list."""
-        return [load_frid_str(line.decode()) for line in val.splitlines()]
+        return [load_frid_str(line.decode(), **self._frid_loader_params)
+                for line in val.splitlines()]
     def _decode_dict(self, val: bytes, /) -> StrKeyMap:
         """Decode the value as the representation for a dict."""
         out = {}
@@ -454,7 +466,7 @@ class BinaryStoreMixin:
             if tab_str:
                 being = FridBeing.parse(val_str)
                 if being is None:
-                    out[key] = load_frid_str(val_str)
+                    out[key] = load_frid_str(val_str, **self._frid_loader_params)
                 elif being:
                     out[key] = PRESENT
                 else:
@@ -468,13 +480,23 @@ class StreamStoreMixin(BinaryStoreMixin, _SimpleBaseStore[bytes]):
 
     Appendable data types includes text, blob, list, and dict.
     """
-    def __init__(self, *, header_head=b"#!", header_link=b"@[", header_tail=b"]\f", **kwargs):
+    class Params(BinaryStoreMixin.Params, total=False):
+        header_head: bytes
+        header_link: bytes
+        header_tail: bytes
+        frid_loader_params: FridLoader.Params
+        frid_dumper_params: FridDumper.Params
+    def __init__(self, **kwargs: Unpack[Params]):
+        header_head = kwargs.pop('header_head', b"#!")
+        header_link = kwargs.pop('header_link', b"@[")
+        header_tail = kwargs.pop('header_tail', b"]\f")
         super().__init__(
             frid_prefix=header_head,
             text_prefix=(header_head + b'text' + header_link),
             blob_prefix=(header_head + b'blob' + header_link),
             list_prefix=(header_head + b'list' + header_link),
             dict_prefix=(header_head + b'dict' + header_link),
+            **kwargs  # type: ignore -- need dict unpacker
         )
         self._header_head = header_head
         self._header_link = header_link
@@ -515,7 +537,8 @@ class StreamStoreMixin(BinaryStoreMixin, _SimpleBaseStore[bytes]):
         return val[self._header_size:]
 
     ModReturnType = tuple[bytes,bool|None]|FridBeing
-    def _add_new(self, old: bytes|MissingType, new: FridValue) -> ModReturnType:
+    def _add_new(self, old: bytes|MissingType,  # type: ignore -- changing return type
+                 new: FridValue) -> ModReturnType:
         """Combines the `new` value into the `old` values depending on the `flags`.
         - Returns either of the following:
             + PRESENT: use the existing value without change.
@@ -537,7 +560,8 @@ class StreamStoreMixin(BinaryStoreMixin, _SimpleBaseStore[bytes]):
         if isinstance(new_val, FridBeing):
             return new_val
         return (new_val, False)
-    def _del_sel(self, val: bytes|MissingType, sel: VStoreSel, /) -> ModReturnType:
+    def _del_sel(self, val: bytes|MissingType,   # type: ignore -- changing return type
+                 sel: VStoreSel, /) -> ModReturnType:
         if val is not MISSING:
             if isinstance(val, Mapping) and self._get_header_type(val) == 'dict':
                 return (self._remove_dict(sel), True)
