@@ -1,7 +1,7 @@
 import asyncio
 from collections.abc import AsyncIterable, Collection, Iterable, Mapping, Sequence
 from logging import error
-from typing import Any, TypeGuard, TypeVar
+from typing import Any, TypeGuard, TypeVar, TypedDict
 
 from sqlalchemy import (
     Engine, Connection, MetaData, Table, Row, Column, ColumnElement, CursorResult,
@@ -11,6 +11,7 @@ from sqlalchemy import (
 from sqlalchemy.ext.asyncio import create_async_engine, AsyncEngine, AsyncConnection
 from sqlalchemy import types
 
+from ..typing import Unpack
 from ..typing import (
     MISSING, BlobTypes, DateTypes, FridArray, FridBeing,
     FridTypeName, FridTypeSize, FridValue, MissingType, StrKeyMap, frid_type_size, get_type_name
@@ -18,8 +19,8 @@ from ..typing import (
 from ..guards import as_kv_pairs, is_frid_array, is_text_list_like
 from ..chrono import datetime, dateonly, timeonly
 from .._basic import frid_mingle
-from .._dumps import dump_frid_str
-from .._loads import load_frid_str
+from .._dumps import dump_frid_str, FridDumper
+from .._loads import load_frid_str, FridLoader
 from .store import AsyncStore, ValueStore
 from .utils import (
     BulkInput, KeySearch, VSPutFlag, VStoreKey, VStoreSel, is_dict_sel, is_list_sel,
@@ -36,48 +37,71 @@ class _SqlBaseStore:
     # https://docs.sqlalchemy.org/en/21/core/pooling.html#disconnect-handling-pessimistic
     # Also see https://stackoverflow.com/questions/55457069
     engine_args = {'pool_pre_ping': True, 'pool_recycle': 300}
-    def __init__(
-            self, table: Table,
-            *, key_fields: Sequence[str]|str|None=None, val_fields: Sequence[str]|str|None=None,
-            frid_field: str|bool=False, text_field: str|bool=False, blob_field: str|bool=False,
-            row_filter: Mapping[str,SqlTypes]|None=None,
-            col_values: Mapping[str,SqlTypes]|None=None,
-            seq_subkey: str|bool=False, map_subkey: str|bool=False,
-    ):
+
+    class Params(TypedDict, total=False):
+        key_fields: Sequence[str]|str
+        val_fields: Sequence[str]|str
+        frid_field: str|bool
+        text_field: str|bool
+        blob_field: str|bool
+        row_filter: Mapping[str,SqlTypes]
+        col_values: Mapping[str,SqlTypes]
+        seq_subkey: str|bool
+        map_subkey: str|bool
+        frid_loader_params: FridLoader.Params
+        frid_dumper_params: FridDumper.Params
+    def __init__(self, table: Table, **kwargs: Unpack[Params]):
         self._table = table
+        row_filter = kwargs.pop('row_filter', None)
+        col_values = kwargs.pop('col_values', None)
         self._where_conds: list[ColumnElement[bool]] = self._build_where(table, row_filter)
         self._insert_data: Mapping[str,SqlTypes] = dict_concat(row_filter, col_values)
         # For keys
-        self._seq_key_col: Column|None = self._find_sub_key_col(table, seq_subkey, True)
-        self._map_key_col: Column|None = self._find_sub_key_col(table, map_subkey, False)
+        self._seq_key_col: Column|None = self._find_sub_key_col(
+            table, kwargs.pop('seq_subkey', False), True
+        )
+        self._map_key_col: Column|None = self._find_sub_key_col(
+            table, kwargs.pop('map_subkey', False), False
+        )
         exclude: list[str] = []
         if self._seq_key_col is not None:
             exclude.append(self._seq_key_col.name)
         if self._map_key_col is not None:
             exclude.append(self._map_key_col.name)
-        self._key_columns: list[Column] = self._find_key_columns(table, key_fields, exclude)
+        self._key_columns: list[Column] = self._find_key_columns(
+            table, kwargs.pop('key_fields', None), exclude
+        )
         exclude.extend(col.name for col in self._key_columns)
         # For values
-        if frid_field is True and text_field is True:
+        if kwargs.get('frid_field') is True and kwargs.get('text_field') is True:
             raise ValueError("frid_field and text_field cannot both be true; use column names")
         if col_values:
             exclude.extend(col_values.keys())
-        self._frid_column: Column|None = self._find_column(table, frid_field, exclude,
-                                                           types.String)
+        self._frid_column: Column|None = self._find_column(
+            table, kwargs.pop('frid_field', False), exclude, types.String
+        )
         if self._frid_column is not None:
             exclude.append(self._frid_column.name)
-        self._text_column: Column|None = self._find_column(table, text_field, exclude,
-                                                           types.String)
+        self._text_column: Column|None = self._find_column(
+            table, kwargs.pop('text_field', False), exclude, types.String
+        )
         if self._text_column is not None:
             exclude.append(self._text_column.name)
-        self._blob_column: Column|None = self._find_column(table, blob_field, exclude,
-                                                           types.LargeBinary)
+        self._blob_column: Column|None = self._find_column(
+            table, kwargs.pop('blob_field', False), exclude, types.LargeBinary
+        )
         if self._blob_column is not None:
             exclude.append(self._blob_column.name)
-        self._val_columns: list[Column] = self._find_val_columns(table, val_fields, exclude)
+        self._val_columns: list[Column] = self._find_val_columns(
+            table, kwargs.pop('val_fields', None), exclude
+        )
+
+        self._frid_loader_params: FridLoader.Params = kwargs.pop('frid_loader_params', {})
+        self._frid_dumper_params: FridDumper.Params = kwargs.pop('frid_dumper_params', {})
         # TODO: if row is autoincrement integer is part of primary key then it is for a list
         # If set to True, find such a column
         # self._multi_rows = table.c[multi_rows] if isinstance(multi_rows, str) else multi_rows
+        super().__init__(**kwargs)  # type: ignore
 
         self._select_cols: list[Column] = self._select_args()
 
@@ -280,7 +304,7 @@ class _SqlBaseStore:
             if not val:
                 return out
         if self._frid_column is not None:
-            out[self._frid_column.name] = dump_frid_str(val)
+            out[self._frid_column.name] = dump_frid_str(val, **self._frid_dumper_params)
             return out
         raise ValueError(f"No column to store data of type {type(val)}")
     def _extract_row_value(
@@ -315,7 +339,7 @@ class _SqlBaseStore:
                 continue
             if self._frid_column is not None and col.name == self._frid_column.name:
                 if val and isinstance(val, str):
-                    frid_val = load_frid_str(val)
+                    frid_val = load_frid_str(val, **self._frid_loader_params)
                 else:
                     error(f"Data in column {col.name} is not types.String: {type(val)}")
                 continue
@@ -644,14 +668,15 @@ class _SqlBaseStore:
 class DbsqlValueStore(_SqlBaseStore, ValueStore):
     def __init__(self, conn_url: str, table: Table, /,
                  *, engine_args: Mapping[str,Any]|None=None, _engine: Engine|None=None,
-                 **kwargs):
+                 **kwargs: Unpack[_SqlBaseStore.Params]):
         eng_args = dict_concat(self.engine_args, engine_args)
         self._engine = create_engine(conn_url, **eng_args) if _engine is None else _engine
-        super().__init__(table=table, **kwargs)
+        super().__init__(table, **kwargs)
 
     @classmethod
     def from_url(cls, url: str, table: Table|str, /,
-                 *, engine_args: Mapping[str,Any]|None=None, **kwargs):
+                 *, engine_args: Mapping[str,Any]|None=None,
+                 **kwargs: Unpack[_SqlBaseStore.Params]):
         """The exmples for URL format for SQL Value Stores are:
         - SQLite: "sqlite+pysqlite:////abs/path/to/file"
         - PostgreSQL: "postgresql+psycopg://postgres:PASSWORD@HOST"
@@ -744,13 +769,14 @@ class DbsqlValueStore(_SqlBaseStore, ValueStore):
 class DbsqlAsyncStore(_SqlBaseStore, AsyncStore):
     def __init__(self, conn_url: str, table: Table, /,
                  *, engine_args: Mapping[str,Any]|None=None, _engine: AsyncEngine|None=None,
-                 **kwargs):
+                 **kwargs: Unpack[_SqlBaseStore.Params]):
         eng_args = dict_concat(self.engine_args, engine_args)
         self._engine = create_async_engine(conn_url, **eng_args) if _engine is None else _engine
         super().__init__(table, **kwargs)
     @classmethod
     async def from_url(cls, url: str, table_name: Table|str, /,
-                       *, engine_args: Mapping[str,Any]|None=None, **kwargs):
+                       *, engine_args: Mapping[str,Any]|None=None,
+                       **kwargs: Unpack[_SqlBaseStore.Params]):
         """The exmples for URL format for SQL Async Stores are:
         - SQLite: "sqlite+aiosqlite:////abs/path/to/file"
           (requires `pip3 install aiosqlite`)
@@ -782,7 +808,7 @@ class DbsqlAsyncStore(_SqlBaseStore, AsyncStore):
 
     def get_lock(self, name: str|None=None):
         raise NotImplementedError
-    async def get_keys(self, pat: KeySearch) -> AsyncIterable[VStoreKey]:
+    async def get_keys(self, pat: KeySearch=None) -> AsyncIterable[VStoreKey]:
         cmd = self._get_keys_select(pat)
         async with self._engine.begin() as conn:
             for x in self._get_keys_result(await conn.execute(cmd), pat):
