@@ -4,7 +4,7 @@ from concurrent.futures import Executor
 from contextlib import AbstractAsyncContextManager
 from typing import Concatenate, ParamSpec, TypeVar
 
-from ..typing import MISSING, BlobTypes, FridBeing, FridTypeName, FridTypeSize, FridValue
+from ..typing import MISSING, MissingType, BlobTypes, FridTypeName, FridTypeSize, FridValue
 from ..aio import gather_aiter
 from . import utils
 from .store import AsyncStore, ValueStore
@@ -30,12 +30,11 @@ class ValueProxyStore(ValueStore):
                  keys: Iterable[VStoreKey]|None=None) -> Mapping[VStoreKey,FridTypeSize]:
         return self._store.get_meta(*args, keys=keys)
     def get_frid(self, key: VStoreKey, sel: VStoreSel=None,
-                 /, dtype: FridTypeName='') -> FridValue|FridBeing:
+                 /, dtype: FridTypeName='') -> FridValue|MissingType:
         return self._store.get_frid(key, sel, dtype)
-    def put_frid(self, key: VStoreKey, val: FridValue,
-                 /, flags=VSPutFlag.UNCHECKED) -> int|bool:
+    def put_frid(self, key: VStoreKey, val: FridValue, /, flags=VSPutFlag.UNCHECKED) -> bool:
         return self._store.put_frid(key, val, flags)
-    def del_frid(self, key: VStoreKey, sel: VStoreSel=None, /) -> int|bool:
+    def del_frid(self, key: VStoreKey, sel: VStoreSel=None, /) -> bool:
         return self._store.del_frid(key, sel)
     def get_bulk(self, keys: Iterable[VStoreKey], /, alt: _T=MISSING) -> list[FridValue|_T]:
         return self._store.get_bulk(keys, alt)
@@ -71,12 +70,12 @@ class AsyncProxyStore(AsyncStore):
                        keys: Iterable[VStoreKey]|None=None) -> Mapping[VStoreKey,FridTypeSize]:
         return await self._store.get_meta(*args, keys=keys)
     async def get_frid(self, key: VStoreKey, sel: VStoreSel=None,
-                       /, dtype: FridTypeName='') -> FridValue|FridBeing:
+                       /, dtype: FridTypeName='') -> FridValue|MissingType:
         return await self._store.get_frid(key, sel, dtype)
     async def put_frid(self, key: VStoreKey, val: FridValue,
-                        /, flags=VSPutFlag.UNCHECKED) -> int|bool:
+                        /, flags=VSPutFlag.UNCHECKED) -> bool:
         return await self._store.put_frid(key, val, flags)
-    async def del_frid(self, key: VStoreKey, sel: VStoreSel=None, /) -> int|bool:
+    async def del_frid(self, key: VStoreKey, sel: VStoreSel=None, /) -> bool:
         return await self._store.del_frid(key, sel)
     async def get_bulk(self, keys: Iterable[VStoreKey],
                        /, alt: _T=MISSING) -> list[FridValue|_T]:
@@ -89,7 +88,7 @@ class AsyncProxyStore(AsyncStore):
         return await self._store.get_text(key, alt)
     async def get_blob(self, key: VStoreKey, alt: _T=None) -> BlobTypes|_T:
         return await self._store.get_blob(key, alt)
-    async def get_list(self, key: VStoreKey, sel: VSListSel,
+    async def get_list(self, key: VStoreKey, sel: VSListSel=None,
                        /, alt: _T=None) -> list[FridValue]|FridValue|_T:
         return await self._store.get_list(key, sel, alt)
     async def get_dict(self, key: VStoreKey, sel: VSDictSel=None,
@@ -127,7 +126,7 @@ class ValueProxyAsyncStore(AsyncStore):
         if depth > 0:
             return await self._asyncrun(self._store.finalize, depth - 1)
     def get_lock(self, name: str|None=None):
-        return self._store.get_lock(name)
+        raise NotImplementedError
     async def get_keys(self, pat: KeySearch=None, /) -> AsyncIterable[VStoreKey]:
         for key in self._store.get_keys(pat):
             yield key
@@ -135,12 +134,12 @@ class ValueProxyAsyncStore(AsyncStore):
                        keys: Iterable[VStoreKey]|None=None) -> Mapping[VStoreKey,FridTypeSize]:
         return await self._asyncrun(self._store.get_meta, *utils.list_concat(args, keys))
     async def get_frid(self, key: VStoreKey, sel: VStoreSel=None,
-                       /, dtype: FridTypeName='') -> FridValue|FridBeing:
+                       /, dtype: FridTypeName='') -> FridValue|MissingType:
         return await self._asyncrun(self._store.get_frid, key, sel, dtype)
     async def put_frid(self, key: VStoreKey, val: FridValue,
-                        /, flags=VSPutFlag.UNCHECKED) -> int|bool:
+                        /, flags=VSPutFlag.UNCHECKED) -> bool:
         return await self._asyncrun(self._store.put_frid, key, val, flags)
-    async def del_frid(self, key: VStoreKey, sel: VStoreSel=None, /) -> int|bool:
+    async def del_frid(self, key: VStoreKey, sel: VStoreSel=None, /) -> bool:
         return await self._asyncrun(self._store.del_frid, key, sel)
     async def get_bulk(self, keys: Iterable[VStoreKey],
                        /, alt: _T=MISSING) -> list[FridValue|_T]:
@@ -153,7 +152,7 @@ class ValueProxyAsyncStore(AsyncStore):
         return await self._asyncrun(self._store.get_text, key, alt)
     async def get_blob(self, key: VStoreKey, alt: _T=None) -> BlobTypes|_T:
         return await self._asyncrun(self._store.get_blob, key, alt)
-    async def get_list(self, key: VStoreKey, sel: VSListSel,
+    async def get_list(self, key: VStoreKey, sel: VSListSel=None,
                        /, alt: _T=None) -> list[FridValue]|FridValue|_T:
         return await self._asyncrun(self._store.get_list, key, sel, alt)
     async def get_dict(self, key: VStoreKey, sel: VSDictSel=None,
@@ -190,6 +189,8 @@ class AsyncProxyValueStore(ValueStore):
     def finalize(self, depth=0):
         if depth > 0:
             result = self._loop.run_until_complete(self._store.finalize(depth - 1))
+        else:
+            result = None
         self._del_loop()
         return result
     def get_keys(self, pat: KeySearch=None, /) -> Iterable[VStoreKey]:
@@ -198,12 +199,11 @@ class AsyncProxyValueStore(ValueStore):
                  keys: Iterable[VStoreKey]|None=None) -> Mapping[VStoreKey,FridTypeSize]:
         return self._loop.run_until_complete(self._store.get_meta(*args, keys=keys))
     def get_frid(self, key: VStoreKey, sel: VStoreSel=None,
-                 /, dtype: FridTypeName='') -> FridValue|FridBeing:
+                 /, dtype: FridTypeName='') -> FridValue|MissingType:
         return self._loop.run_until_complete(self._store.get_frid(key, sel, dtype))
-    def put_frid(self, key: VStoreKey, val: FridValue,
-                 /, flags=VSPutFlag.UNCHECKED) -> int|bool:
+    def put_frid(self, key: VStoreKey, val: FridValue, /, flags=VSPutFlag.UNCHECKED) -> bool:
         return self._loop.run_until_complete(self._store.put_frid(key, val, flags))
-    def del_frid(self, key: VStoreKey, sel: VStoreSel=None, /) -> int|bool:
+    def del_frid(self, key: VStoreKey, sel: VStoreSel=None, /) -> bool:
         return self._loop.run_until_complete(self._store.del_frid(key, sel))
     def get_bulk(self, keys: Iterable[VStoreKey], /, alt: _T=MISSING) -> list[FridValue|_T]:
         return self._loop.run_until_complete(self._store.get_bulk(keys, alt))
