@@ -1,20 +1,26 @@
+import os, sys, traceback, dataclasses
 from functools import partial
-import os, traceback, dataclasses
 from abc import ABC
 from datetime import date as dateonly, time as timeonly, datetime
 from collections.abc import Mapping, Sequence, Set, Callable
 from enum import Enum
 from types import TracebackType
 from typing import Any, Generic, Literal, NamedTuple, TypeGuard, TypeVar, final
+if sys.version_info >= (3, 11):
+    from typing import NotRequired, Required, Unpack
+else:
+    from typing_extensions import NotRequired, Required, Unpack  # noqa: F401
 
-
-FRID_ERROR_VENUE = os.getenv('FRID_ERROR_VENUE')
+# === Generic type definitions ===
 
 # Quick union types used in many places
 BlobTypes = bytes|bytearray|memoryview
 DateTypes = dateonly|timeonly|datetime   # Note that datetime in Python is deriveed from date
 
-# FRID types follow (Flexibly represented inteactive data)
+# === FRID types follow (Flexibly represented inteactive data) ===
+
+# This is a configurable value to be assigned to the "venue" field of HttpError
+FRID_ERROR_VENUE = os.getenv('FRID_ERROR_VENUE')
 
 _T = TypeVar('_T')
 _B = TypeVar('_B', bound='FridBasic')
@@ -94,10 +100,10 @@ class FridMixin(ABC):
         return [cls.__name__]
 
     @classmethod
-    def frid_from(cls: type[_M], data: 'FridNameArgs|FridArray|StrKeyMap', /, **kwargs) -> _M:
+    def frid_from(cls: type[_M], data: 'FridNameArgs', /, **kwargs) -> _M:
         """Construct an instance with given name and arguments.
         - `data`: An array for a positional arguments or a map for keywords
-          arguemtns, or a combination of them in the type of `FridNameArgs`.
+          arguments, or a combination of them in the type of `FridNameArgs`.
         - `dc_ignore_extra` (only effective for dataclasses): if set, ignore
           extra positional arguments and non-matching keyward arguments;
           if set to a callback function, also call this function for each of
@@ -330,7 +336,7 @@ class FridError(FridMixin, Exception):
             raise ValueError(f"Invalid trace type {type(trace)}")
 
     @classmethod
-    def frid_from(cls, data: FridNameArgs, /):
+    def frid_from(cls, data: FridNameArgs, /, **kwargs):
         # The `trace` and `cause` are not accepting TrackbackType and BaseException;
         # and `error` is passed as the first argument.
         assert data.name in cls.frid_keys()
