@@ -4,18 +4,17 @@ from dataclasses import dataclass
 from collections.abc import AsyncIterable, Iterable, Mapping, Callable, Sequence
 from typing import Any, Literal, TypedDict
 from functools import partial
-if sys.version_info >= (3, 11):
-    from typing import NotRequired, Unpack
-else:
-    from typing_extensions import NotRequired, Unpack  # noqa: F401
+from fnmatch import fnmatch
 
-from ..osutil import load_module_data
-from ..typing import get_type_name, get_func_name
+from ..typing import Unpack   # Python 3.11 only feature
 from ..typing import FridNameArgs, FridValue, MissingType, MISSING
+from ..typing import get_type_name, get_func_name
 from ..guards import is_frid_value
+from ..lib import CaseDict
 from .._basic import frid_redact
 from .._dumps import dump_args_str
-from .mixin import HttpError, HttpMixin, InputHttpHead, parse_url_query, parse_url_value
+from .._utils import load_module_data
+from .mixin import HttpError, HttpMixin, HttpInputHead, parse_url_query, parse_url_value
 from .files import FileRouter
 
 
@@ -35,17 +34,18 @@ _api_call_types: dict[str,ApiCallType] = {
 HTTP_SUPPORTED_METHODS = ('HEAD', 'GET', 'PUT', 'POST', 'DELETE', 'PATCH')
 HTTP_METHODS_WITH_BODY = ('POST', 'PUT', 'PATCH')
 
-class HttpInfo(TypedDict, total=False):
+class HttpInput(TypedDict, total=False):
     path: str               # The path to in the URL
     qstr: str               # The query string in the URL
     frag: list[str]         # Three element list after path fragmentation
     call: ApiCallType       # One of the five calls
-    head: dict[str,str]     # The headers of the call
+    head: Mapping[str,str]  # The headers of the call
     mime: str               # The mime-type of the body
     body: bytes             # The body of the call
     data: FridValue         # The data of the call
     auth: str               # The auth string from the header or elsewhere
     peer: str               # The peer IP address (no port)
+    want: Sequence[str]     # A list of MIME types for most wanted to least wanted
 
 @dataclass
 class ApiRoute:
@@ -88,10 +88,10 @@ class ApiRoute:
     kwargs: dict[str,FridValue]
     numfpa: Literal[0,1,2]
 
-    def __call__(self, req: HttpMixin, **kwargs: Unpack[HttpInfo]):
+    def __call__(self, req: HttpMixin, **kwargs: Unpack[HttpInput]):
         # Fetch authorization status
         peer = kwargs.get('peer')
-        auth = req.http_head.get('authorization')
+        auth = req.http_head.get('Authorization')
         if isinstance(auth, str):
             pair = auth.split()
             if len(pair) == 2 and pair[0] == "Bearer":
@@ -103,24 +103,28 @@ class ApiRoute:
         info(msg)
         # Generates the HttpInfo structure users might need
         assert not isinstance(req.http_data, AsyncIterable)
-        http_info: HttpInfo = {
-            'call': _api_call_types[self.method], **kwargs, 'head': req.http_head,
+        http_input: HttpInput = {
+            'call': _api_call_types[self.method], **kwargs,
+            'head': req.http_head,
             'frag': self.pfrags,
+            'want': [
+                x.split(';')[0].strip() for x in accept.split(',') if x.strip()
+            ] if (accept := req.http_head.get('Accept')) is not None else []
         }
         if req.http_data is not MISSING:
-            http_info['data'] = req.http_data
+            http_input['data'] = req.http_data
         if req.mime_type is not None:
-            http_info['mime'] = req.mime_type
+            http_input['mime'] = req.mime_type
         if req.http_body is not None:
             assert not isinstance(req.http_body, AsyncIterable)
-            http_info['body'] = req.http_body
+            http_input['body'] = req.http_body
         if auth is not None:
-            http_info['auth'] = auth
+            http_input['auth'] = auth
         try:
             args = self._get_vpargs(req.http_data)
             kwds = self._get_kwargs(req.http_data)
             try:
-                return self.action(*args, **kwds, _http=http_info)
+                return self.action(*args, **kwds, _http=http_input)
             except TypeError:
                 pass
             return self.action(*args, **kwds)
@@ -187,6 +191,7 @@ class ApiRouteManager:
         + A map from paths to URL path prefixes.
       For each unique prefixes, a single file router is created.
     - `accept_origins`: the list of origins that can be accepted.
+      One can use hostname only (no https:// or http://) and use glob patterns.
       The header 'Access-Control-Allow-Origin' is set if the origin is in the list.
     - `set_connection`: if not None, `Connection: keep-alive` (for true value) or
       `Connection: close` (for false value) is added to the header.
@@ -217,6 +222,9 @@ class ApiRouteManager:
         # 'Connection': "keep-alive",
         'Content-Encoding': "none",
     }  # TODO: add CORS & cache constrol headers
+    _localhost_list = [
+        "localhost", "127.0.0.1", "[::1]",
+    ]
 
     @classmethod
     def __init_subclass__(cls):
@@ -440,16 +448,32 @@ class ApiRouteManager:
                 "X-Requested-With, Content-Type, Authorization, Accept",
             'Access-Control-Max-Age': "1728000",
         })
+    def origin_allowed(self, origin: str) -> bool:
+        if not self.accept_origins:
+            return False
+        origin_no_scheme = split[1] if len(split := origin.split("://")) == 2 else origin
+        for pat in self.accept_origins:
+            if not pat:
+                continue
+            name = origin if "://" in pat else origin_no_scheme
+            # Patter can not be starting with [ or ending with ] because it is IPv6 host
+            if '*' in pat or '?' in pat or ('[' in pat[1:] and ']' in pat[:-1]):
+                if fnmatch(name, pat):
+                    return True
+            else:
+                if pat == name:
+                    return True
+        return False
     def update_headers(self, response: HttpMixin, request: HttpMixin):
         """Adding extra headers to response; mostly for CORS, cache, and access control."""
         headers = response.http_head
         headers.update(self._common_headers)
-        host = request.http_head.get('host')
+        host = request.http_head.get('Host')
         assert isinstance(host, str)
         if ':' in host:
             host = host.split(':')[0]
-        origin = request.http_head.get('origin')
-        if origin and (origin in self.accept_origins or host in ('127.0.0.1', 'localhost')):
+        origin = request.http_head.get('Origin')
+        if origin and (host in self._localhost_list or self.origin_allowed(origin)):
             headers['Access-Control-Allow-Origin'] = origin
         if isinstance(response.http_data, AsyncIterable):
             headers['Access-Control-Allow-Credentials'] = "true"
@@ -460,7 +484,7 @@ class ApiRouteManager:
         return headers
 
     def handle_request(
-            self, method: HttpMethod, data: bytes|None, headers: InputHttpHead,
+            self, method: HttpMethod, data: bytes|None, headers: HttpInputHead,
             *, path: str, qstr: str|None, peer: str|tuple[str,int]|None,
     ) -> tuple[HttpMixin,HttpMixin|FridValue]:
         """Create a request object and run the route.
@@ -482,7 +506,7 @@ class ApiRouteManager:
         route = self.create_route(method, path, qstr)
         if isinstance(route, HttpError):
             return (request, route)
-        kwargs: HttpInfo = {'path': path}
+        kwargs: HttpInput = {'path': path}
         if qstr is not None:
             kwargs['qstr'] = qstr
         if peer is not None:
@@ -504,7 +528,7 @@ class ApiRouteManager:
             response = result
         else:
             ht_status = 200
-            http_head: dict[str,str] = {}
+            http_head = CaseDict[str,str]()
             mime_type: str|None = None
             if isinstance(result, tuple):
                 if not 2 <= len(result) <= 3:
@@ -523,7 +547,7 @@ class ApiRouteManager:
         return response
 
 def echo_router(*args, _data: FridValue|MissingType=MISSING,
-                _call: str='get', _http: HttpInfo={}, **kwds):
+                _call: str='get', _http: HttpInput={}, **kwds):
     args = list(args)
     if _call == 'get':
         if not kwds:
